"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const databases_1 = require("./utils/databases");
const Fastify = tslib_1.__importStar(require("fastify"));
const elastic_apm_node_1 = tslib_1.__importDefault(require("elastic-apm-node"));
const rabbit_1 = tslib_1.__importDefault(require("./utils/rabbit"));
const logger_1 = tslib_1.__importDefault(require("./utils/logger"));
class Microservice {
    constructor(config) {
        this.agent = elastic_apm_node_1.default.start({
            logLevel: 'trace'
        });
        // connects to database
        if (config.databases) {
            databases_1.initDbInstances(this, config.databases);
        }
        // connects to rabbitmq
        if (config.rabbit) {
            this.rabbit = new rabbit_1.default('amqp://localhost');
        }
        //init MicroService
        this.server = Fastify.fastify({
            logger: true
        });
        this.config = config;
        this.logger = new logger_1.default();
    }
    declareRoute(verb, path, handler, options) {
        /**
         * creates an endpoint in the microservice with configs
         *
         * @param verb - http restful methods
         * @param path - path of endpoint
         * @param handler - function that handles incoming requests
         * @param options - additional options to configure route
         *
         */
        let onRequest = null;
        if (options && options.onRequest)
            onRequest = options.onRequest;
        const apmContextInject = async (req, res) => {
            this.agent.setCustomContext(req);
        };
        if (onRequest) {
            onRequest =
                typeof onRequest === 'function'
                    ? [apmContextInject, onRequest]
                    : [apmContextInject, ...onRequest];
        }
        else {
            onRequest = [apmContextInject];
        }
        this.server.route({
            ...options,
            method: verb,
            url: path,
            handler: handler,
            onRequest: onRequest
        });
    }
    publish(msg, queue) {
        /**
         * publishes a message to a RabbitMQ queue
         *
         * @param msg - message to send to rabbit
         * @param queue - queue the message is supposed to go
         *
         */
        if (this.rabbit)
            this.rabbit.pub(queue, msg);
    }
    log(log) {
        /**
         * logs a statement
         *
         * @param log - statement to be logged
         *
         */
        this.logger.log(log);
    }
    error(log, error) {
        /**
         * logs an error
         *
         * @param log - error message to be logged
         * @param error - error
         *
         */
        this.logger.error(log, error);
    }
    async start() {
        /**
         * starts the microservice
         *
         * @return boolean of starting microservice
         */
        return await this.server
            .listen(this.config.port)
            .then(() => {
            console.log('SERVER LISTENING');
            return true;
        })
            .catch((err) => {
            console.log(err);
            return false;
        });
    }
}
exports.default = Microservice;
