"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const amqp = tslib_1.__importStar(require("amqplib"));
const logger_1 = tslib_1.__importDefault(require("./logger"));
class rabbit {
    constructor(url) {
        this.STATUS = 'CONNECTING';
        this.messageStore = [];
        this.connection = null;
        this.connectionPromise = this.connector(url);
        this.logger = new logger_1.default();
    }
    connector(url) {
        /**
         * initiates the connector to RabbitMQ.
         * It will sleep and attempt re reconnect if it loses or fails to connect
         * On connection, will send all cached messages in messageStore
         *
         * @param url - address of rabbitmq
         *
         */
        return (Promise.resolve()
            .then(() => {
            //can add url formatting logic here
            if (url !== 'amqp://localhost') {
                url = 'amqp://localhost';
            }
            return url;
        })
            .then((url) => {
            return amqp.connect(url).then((conn) => {
                this.logger.log('[AMQP] Connected');
                this.STATUS = 'CONNECTED';
                this.connection = conn;
                this.sendMessageQueue();
                // if the connection closes for some reason
                conn.on('close', (err) => {
                    this.connection = null;
                    this.STATUS = 'RECONNECTING';
                    this.logger.error('[AMQP] lost connection');
                    setTimeout(() => {
                        this.logger.log('[AMQP] attempting to reconnect');
                        this.connector(url);
                    }, 5000);
                });
            });
        })
            // failed to connect
            .catch((err) => {
            this.logger.log(`[AMQP] ${err}`);
            this.connectionPromise = null;
            this.connection = null;
            setTimeout(() => {
                this.logger.log('[AMQP] attempting to connect');
                this.STATUS = 'RECONNECTING';
                this.connector(url);
            }, 5000);
        }));
    }
    sendMessageQueue() {
        /**
         * sends messages that were cached when rabbit not connected
         */
        this.messageStore.forEach((msg) => {
            this.pub(msg.Queue, msg.Msg);
            // TODO: pop out messages if successful
        });
    }
    getStatus() {
        /**
         * gets status of rabbit connection
         */
        return this.STATUS;
    }
    getMessageStore() {
        /**
         * gets messages in message queue
         */
        return this.messageStore;
    }
    async pub(Queue, Msg) {
        /**
         * publishes message to RabbitMQ
         * if fail to publish, send it to messageStore
         *
         * @param Queue - the queue to publish to
         * @param Msg - message of the publication
         *
         */
        if (this.connection !== null) {
            this.connection
                .createChannel()
                .then((ch) => {
                ch.assertQueue(Queue).then((ok) => {
                    ch.sendToQueue(Queue, Buffer.from(Msg));
                    this.logger.log(`[AMQP] PUB ${Queue} : ${Msg}`);
                    return ch.close();
                });
            })
                // if it fails to pub, send to messageStore
                .catch((err) => {
                this.logger.error(`[AMQP] connection - ${err}`);
                this.messageStore.push({ Queue: Queue, Msg: Msg });
            });
        }
        else {
            // if it fails to pub, send to messageStore
            this.logger.error(`[AMQP] NO CONNECTION`);
            this.messageStore.push({ Queue: Queue, Msg: Msg });
        }
    }
    async sub(Queue, callback) {
        /**
         * subscribes to RabbitMQ queue
         *
         * @param Queue - the queue to subscribe to
         * @param callback - handler of the message
         *
         */
        if (this.connection !== null) {
            this.connection
                .createChannel()
                .then((ch) => {
                ch.assertQueue(Queue).then((ok) => {
                    ch.consume(Queue, (msg) => {
                        if (msg !== null) {
                            callback(msg.content.toString());
                            ch.ack(msg);
                        }
                        else {
                            this.logger.log(`[AMQP] MSG EMPTY`);
                        }
                    });
                });
            })
                .catch((err) => {
                this.logger.error(`[AMQP] connection - ${err}`);
            });
        }
        else {
            this.logger.error(`[AMQP] NO CONNECTION`);
        }
    }
    async createQueue(queue) {
        /**
         * creates a RabbitMQ queue
         *
         * @param queue - name of the queue
         *
         */
        if (this.connection !== null) {
            this.connection.createChannel().then((ch) => {
                ch.assertQueue(queue)
                    .then((ok) => {
                    this.logger.log(`[AMQP] CREATED QUEUE : ${queue}`);
                })
                    .catch((err) => {
                    this.logger.error(`[AMQP] ${err}`);
                });
            });
        }
        else {
            this.logger.error(`[AMQP] NO CONNECTION`);
        }
    }
    async deleteQueue(queue) {
        /**
         * deletes a RabbitMQ queue
         *
         * @param queue - name of the queue
         *
         */
        if (this.connection !== null) {
            this.connection.createChannel().then((ch) => {
                ch.deleteQueue(queue)
                    .then((ok) => {
                    this.logger.log(`[AMQP] Deleted QUEUE : ${queue}`);
                })
                    .catch((err) => {
                    this.logger.error(`[AMQP] ${err}`);
                });
            });
        }
        else {
            this.logger.error(`[AMQP] NO CONNECTION`);
        }
    }
    async createExchange(name, type) {
        /**
         * create a RabbitMQ exchange.
         *
         * @param name - name of the exchange
         * @param type - type of the exchange
         *
         */
        if (this.connection !== null) {
            this.connection.createChannel().then((ch) => {
                ch.assertExchange(name, type, { durable: false })
                    .then((ok) => {
                    this.logger.log(`[AMQP] CREATED EXCHANGE : ${name}`);
                })
                    .catch((err) => {
                    this.logger.error(`[AMQP] ${err}`);
                });
            });
        }
        else {
            this.logger.error(`[AMQP] NO CONNECTION`);
        }
    }
    async deleteExchange(name) {
        /**
         * deletes a RabbitMQ exchange.
         *
         * @param name - name of the exchange
         *
         */
        if (this.connection !== null) {
            this.connection.createChannel().then((ch) => {
                ch.deleteExchange(name)
                    .then((ok) => {
                    this.logger.log(`[AMQP] DELETED EXCHANGE : ${name}`);
                })
                    .catch((err) => {
                    this.logger.error(`[AMQP] ${err}`);
                });
            });
        }
        else {
            this.logger.error(`[AMQP] NO CONNECTION`);
        }
    }
    bindQueue(
    /**
     * binds an exchange to a queue
     *
     * @param queueName - name of the queue
     * @param exchangeName - name of the exchange
     * @param pattern - ???
     * @param args - additional args
     *
     */
    queueName, exchangeName, pattern, args) {
        if (this.connection !== null) {
            this.connection.createChannel().then((ch) => {
                ch.bindQueue(queueName, exchangeName, '', args)
                    .then((ok) => {
                    this.logger.log(`[AMQP] BINDED QUEUE ${queueName} TO EXCHANGE ${exchangeName}`);
                })
                    .catch((err) => {
                    this.logger.error(`[AMQP] ${err}`);
                });
            });
        }
        else {
            this.logger.error(`[AMQP] NO CONNECTION`);
        }
    }
}
exports.default = rabbit;
